/*
 * jammo_singinggame.c
 *
 * This file is part of JamMo.
 *
 * (c) 2009-2010 University of Oulu
 *
 * Authors: Aapo Rantalainen
 */

#include <tangle.h>
#include <clutter/clutter.h>
#include <unistd.h>
#include <fcntl.h> //we check some files
#include "string.h"

#include "../chum/chum.h"
#include "../chum/gui.h" /*For SCREEN_*-macros */

#include "../meam/jammo-recording-track.h"
#include "jammo-mentor.h"
#include "jammo-cursor.h"
#include "../meam/jammo-playing-track.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
static JammoSequencer* singinggame_sequencer = NULL; //used for stopping


static ClutterActor *contour_cursor;
static ClutterActor *contour_trace;
static ClutterActor *singinggame_display_freq;


static void add_cursor(ClutterColor* color, JammoSequencer* sequencer, ClutterActor* parent) {
	ClutterActor *container = tangle_widget_new();
	ClutterActor* texture_color;
	ClutterActor* texture_image;
	ClutterActor* cursor;

	//Try to use song's own cursor.png
	gchar* filepath1 = g_strdup_printf("%s/songs/%s/cursor.png", DATA_DIR,chum_get_current_song());
	int fd = open(filepath1,O_RDONLY);
	if (fd == -1) //File not found
		{
		printf("file '%s' not found\n",filepath1);
		ClutterColor transparency = { 0, 0, 0, 0 };
		texture_image = clutter_rectangle_new_with_color(&transparency);
		clutter_actor_set_size(texture_image, 60.0,60.0);
		}
	else
		{
		texture_image = clutter_texture_new();
		clutter_texture_set_from_file (CLUTTER_TEXTURE (texture_image), filepath1, NULL);
		}
	g_free(filepath1);
	
	gfloat width,height;
	clutter_actor_get_size(texture_image, &width, &height);
	
	texture_color = clutter_rectangle_new_with_color(color);
	clutter_actor_set_size(texture_color, width+4,height+4); //slightly bigger than image
	
	tangle_widget_add(TANGLE_WIDGET(container), texture_color, NULL);
	tangle_widget_add(TANGLE_WIDGET(container), texture_image, NULL);
	
	
	cursor = jammo_cursor_new(sequencer, container);
	tangle_actor_set_depth_position(TANGLE_ACTOR(cursor), 1);
	clutter_actor_set_position(cursor, 0.0, 480.0-height-2);
	clutter_actor_set_size(cursor, 800.0, height+4);
	clutter_container_add_actor(CLUTTER_CONTAINER(parent), cursor);
}

/*
This will be more fancier some day.
*/
#define radius  5.0
static void draw_trace(int x, int y) {
	//offset: it looks like contour_star (png at this moment) is drawing line
	cogl_path_ellipse (x+5, y+15, radius, radius);
	cogl_path_stroke();
}

static void update_trace(ClutterActor *data,int x,int y){
		//printf("update_texture called\n");
		CoglHandle handle;
		CoglHandle offscreen;

		if ((handle = clutter_texture_get_cogl_texture(CLUTTER_TEXTURE(data))) == COGL_INVALID_HANDLE) 
		{
					handle = cogl_texture_new_with_size(800, 480, COGL_TEXTURE_NO_SLICING, COGL_PIXEL_FORMAT_RGBA_8888_PRE);
					g_assert(handle != COGL_INVALID_HANDLE);
					clutter_texture_set_cogl_texture(CLUTTER_TEXTURE(data), handle);
		}

		cogl_push_draw_buffer();
		
		/* TODO: the following memory allocation leaks even if there is unref. */
		offscreen = cogl_offscreen_new_to_texture(handle);

		g_assert(offscreen != COGL_INVALID_HANDLE);
		cogl_set_draw_buffer(COGL_OFFSCREEN_BUFFER, offscreen);

		cogl_set_source_color4f(1.0, 1.0, 0.1, 1.0);

		draw_trace(x,y);
		cogl_pop_draw_buffer();	
		cogl_offscreen_unref(offscreen);

		clutter_actor_queue_redraw(CLUTTER_ACTOR(data));
}


#define LEFT_MARGINAL 10      /*Marginal from left border.*/
int contour_counter=LEFT_MARGINAL; /* This counts how many times cursor is moved*/
/*
FIXME we should handle x-axis with duration + current place of playback track.
Now we just keep count how many signals are cathed.
*/
static void on_pitch_detected(JammoRecordingTrack* track, gfloat frequency, gpointer user_data) {
		g_return_if_fail(contour_cursor != NULL);
		g_return_if_fail(contour_trace != NULL);

		contour_counter+=1;
		//int Y=(contour_counter-400)*(contour_counter-400)*0.001+100; //This is good for demo 
		int Y=840-(frequency*2); //frequence [220,480] -> place [480,0] //origo is on upper left corner

		//if (contour_counter%10==0) //If it takes too much effort to draw it in each round.
		update_trace(contour_trace,contour_counter,Y);
		clutter_actor_set_position(contour_cursor,contour_counter,Y);

		
		if (frequency==-1.0) /*This means, we do not have good value*/
			clutter_text_set_text (CLUTTER_TEXT(singinggame_display_freq),"[ ∞ ]");
		else
			{
			gchar* buffer0;
			buffer0 = g_strdup_printf("[%3.1lf]",frequency);
			clutter_text_set_text (CLUTTER_TEXT(singinggame_display_freq),buffer0);
			g_free(buffer0);
			}
		
}



/*
This function is globally visible.
Whenever 'cross-icon' is pressed, this sequencer is stopped.
*/
void singinggame_start_sequencer(JammoSequencer* sequencer) {
	singinggame_sequencer = sequencer;
	jammo_sequencer_play(sequencer);
}


static void start_sequencer(JammoMentor* mentor, const gchar* speech, gpointer sequencer){
	clutter_actor_hide(CLUTTER_ACTOR(jammo_mentor_get_default()));
	singinggame_start_sequencer(JAMMO_SEQUENCER(sequencer));
}


static void song_view_showed(TangleActor *actor, ClutterActorBox *arg1, ClutterActorBox *arg2, gpointer sequencer){
	printf("song_view_showed\n");

	gchar *filepath1;
	filepath1 =  g_strdup_printf("/opt/jammo/mentor_speech/mentor_song_%s_fi.ogg", chum_get_current_song());
	//printf("pass to mentor: '%s' \n",filepath1);

	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), filepath1, start_sequencer, sequencer);
	jammo_mentor_set_idle_speech(jammo_mentor_get_default(), "");
	g_free(filepath1);
}



static void on_sequencer_stopped_playback(JammoSequencer* sequencer_old, gpointer none) {
	printf("Sequencer stopped_playback\n");

	//Hide all
	ClutterActor*	mainview = jammo_get_actor_by_id("main-views-widget"); //No drawing on stage anymore, use this.
	clutter_container_foreach(CLUTTER_CONTAINER(mainview), chum_hide_this_tangle_actor, NULL);
	
	//Go back to songselection
	ClutterActor* view = jammo_get_actor_by_id("songselection-view");
	clutter_actor_show(view);

	//And show mentor again
	tangle_actor_show(TANGLE_ACTOR(jammo_mentor_get_default()));


	//Automatic saving. THIS MUST BE DOCUMENTED.

	//Saving to cupboard and to log.
	gchar* cmd;

	//TODO: save selected_language to filename too
	//Copy that recorded song to 'singings' and name it.
	cmd = g_strdup_printf("cp %s/last_recorded.wav %s/%s.wav", JAMMO_DIRECTORY, SINGINGS_DIRECTORY,chum_get_current_song() );
	if (system(cmd)) 
		printf("Error, can't call '%s'\n",cmd);
	g_free(cmd);

	//And make it permanent copy for carer/researcher. Filenames look like this: log_singing_2010.02.22_21.16.27.wav
	//Use mv so it is faster (and last_recorded should not be used anymore)
	cmd = g_strdup_printf("mv %s/last_recorded.wav %s/log_singing_`date +%%Y.%%m.%%d_%%H.%%M.%%S`.wav",JAMMO_DIRECTORY,JAMMO_DIRECTORY);
	if (system(cmd)) 
		printf("Error, can't call '%s'\n",cmd);

	g_free(cmd);
}



static void on_sequencer_stopped_recording(JammoSequencer* sequencer_old, gpointer user_data) {
	ClutterActor* view;
	
	view = CLUTTER_ACTOR(user_data);

	printf("Sequencer stopped_recording\n");

	
	#ifndef OFFICIAL_RELEASE
	clutter_actor_hide(contour_cursor);
	clutter_actor_hide(contour_trace);
	clutter_actor_hide(singinggame_display_freq);
	contour_counter=LEFT_MARGINAL;
	#endif



	//make new sequencer and add tracks to it
	JammoSequencer* sequencer = jammo_sequencer_new();
	g_signal_connect(sequencer, "stopped", G_CALLBACK(on_sequencer_stopped_playback), NULL);

	gchar *filename;
	int fd;
	filename = g_strdup_printf("%s/songs/%s/comping%s.ogg", DATA_DIR,chum_get_current_song(),chum_get_selected_language());
	fd = open(filename,O_RDONLY);
	if (fd == -1) //File not found
		{  //no track for this language,. use  common comping.
		g_free(filename);
		filename = g_strdup_printf("%s/songs/%s/comping.ogg", DATA_DIR,chum_get_current_song()); //try without language postfix
		}
	printf("looking for file '%s' for comping_track\n",filename);

	JammoPlayingTrack* playing_track;  
	playing_track = jammo_playing_track_new(filename);   //for comping track
	jammo_sequencer_add_track(sequencer, JAMMO_TRACK(playing_track));
	g_free(filename);


	gchar *loaded_filename;
	loaded_filename =  g_strdup_printf("%s/last_recorded.wav",JAMMO_DIRECTORY);
	printf("looking for file '%s' for loaded_track\n",loaded_filename);	


	//Recorded file must be there and it must not be zero sized.
	struct stat buf;
	if (stat(loaded_filename, &buf) == -1)
		{
		printf("Recorded file '%s' doesn't found\n",loaded_filename);
		}
	else
		{
		if (buf.st_size == 0)
			{
			unlink(loaded_filename);
			}
		else
			{
			JammoPlayingTrack* playing_track2;
			playing_track2 = jammo_playing_track_new(loaded_filename);
			jammo_sequencer_add_track(sequencer, JAMMO_TRACK(playing_track2));
			}
		}

	g_free(loaded_filename);

	ClutterColor green = { 0, 255, 0, 128 };
	add_cursor(&green, sequencer, view);


	chum_set_state_of_chum(SCREEN_PLAY_RECORDED);

	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "/opt/jammo/mentor_speech/mentor_welldone_fi.ogg", start_sequencer, sequencer);
}


/*
trigger="clicked"
Source= TangleButton.
*/
void singinggame_song_click(TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
	ClutterActor* ct =CLUTTER_ACTOR(source);
	const gchar* name = clutter_actor_get_name(ct);
	printf("Song clicked. get_name: '%s'\n",name);

	const char *song_name=name;
	
	chum_set_current_song(song_name);
	

	gchar* name_of_view;
	name_of_view = g_strdup_printf("song-%s-view", name );
	ClutterActor* view = jammo_get_actor_by_id(name_of_view);
	g_free(name_of_view);


	JammoSequencer* sequencer = jammo_sequencer_new();
	g_signal_connect(sequencer, "stopped", G_CALLBACK(on_sequencer_stopped_recording),view);

	gchar *filename;
	int fd;
	if (chum_get_duetto())
		{
		filename = g_strdup_printf("%s/songs/%s/vocal%s.ogg", DATA_DIR,song_name,chum_get_selected_language());
		fd = open(filename,O_RDONLY);
		if (fd == -1) //File not found
			{  //no vocal track. use only comping.
			filename = g_strdup_printf("%s/songs/%s/comping.ogg", DATA_DIR,song_name); //try without language postfix
			}
		}
	else //solo
	{
	filename = g_strdup_printf("%s/songs/%s/comping%s.ogg", DATA_DIR,song_name,chum_get_selected_language());
	fd = open(filename,O_RDONLY);
			if (fd == -1) //File not found
			{  //no track for this language,. use  common comping.
			filename = g_strdup_printf("%s/songs/%s/comping.ogg", DATA_DIR,song_name); //try without language postfix
			}
	}


	JammoPlayingTrack* playing_track;
	printf("looking for file '%s' \n",filename);
	playing_track = jammo_playing_track_new(filename);   //for comping track
	jammo_sequencer_add_track(sequencer, JAMMO_TRACK(playing_track));

	gchar* recording_filename = g_strdup_printf("%s/last_recorded.wav",JAMMO_DIRECTORY);
	JammoRecordingTrack* recording_track;
	//Do not use pitch_detect on OFFICIAL_RELEASE (not robust enough)
	#ifdef OFFICIAL_RELEASE
	recording_track = jammo_recording_track_new(recording_filename);
	#else
	recording_track = jammo_recording_track_new_with_pitch_detect(recording_filename);
	#endif
	g_free(recording_filename);

	jammo_sequencer_add_track(sequencer, JAMMO_TRACK(recording_track));

	chum_set_state_of_chum(SCREEN_SONG_RECORDING);
	g_free(filename);



	//g_signal_connect(view, "transition-completed", G_CALLBACK(song_view_showed), sequencer); FIXME: this needs modifications to json
	song_view_showed(NULL,NULL,NULL,sequencer);

	//Do not use pitch_detect on OFFICIAL_RELEASE (not robust enough)
	#ifndef OFFICIAL_RELEASE
	const gchar* selected_language=chum_get_selected_language();
	gchar *filepath1;
	//contour_cursor = moving thing
	contour_cursor = clutter_texture_new();
	filepath1 = g_strdup_printf("%s/songs/%s/cursor%s.png", DATA_DIR, name, selected_language );
	fd = open(filepath1,O_RDONLY);
	if (fd == -1) {
		g_free(filepath1);
		filepath1 = g_strdup_printf("%s/songs/%s/cursor.png", DATA_DIR,name ); //try without language postfix
		int fd = open(filepath1,O_RDONLY);
		if (fd == -1) {
			printf("file '%s' not found, using default cursor\n",filepath1);
			//contour_cursor = config_get_configured_actor(NULL,"contour_star");
		} else {
			clutter_texture_set_from_file (CLUTTER_TEXTURE (contour_cursor), filepath1, NULL);
			close(fd);
		}
	} else {
		clutter_texture_set_from_file (CLUTTER_TEXTURE (contour_cursor), filepath1, NULL);
		close(fd);
	}
	g_free(filepath1);

	clutter_container_add_actor (CLUTTER_CONTAINER (view), contour_cursor);



	//contour_trace
	contour_trace = g_object_new (CLUTTER_TYPE_TEXTURE, "disable-slicing", TRUE, NULL);

	CoglHandle new_texture;
	CoglTextureFlags flags = COGL_TEXTURE_NO_SLICING; //NO_SLICING is important.
	new_texture = cogl_texture_new_with_size (WIDTH_OF_WINDOW,HEIGHT_OF_WINDOW,
                                              flags,
                                              COGL_PIXEL_FORMAT_RGBA_8888_PRE);

	clutter_texture_set_cogl_texture (CLUTTER_TEXTURE(contour_trace), new_texture);
	cogl_handle_unref (new_texture);

 clutter_container_add_actor (CLUTTER_CONTAINER (view), contour_trace);

 
  //And for debugging, text-field
	singinggame_display_freq=  clutter_text_new_with_text ("Liberty","START"); //start with empty text
	clutter_text_set_font_name(CLUTTER_TEXT(singinggame_display_freq),"Luxi Mono 18");
	clutter_container_add_actor (CLUTTER_CONTAINER (view), singinggame_display_freq);
	clutter_actor_set_position (singinggame_display_freq, 100, 30); 
	clutter_actor_show (singinggame_display_freq);


	//recording_track is from chum.c. on_pitch_detected uses only globals
	g_signal_connect(recording_track, "pitch-detected", G_CALLBACK(on_pitch_detected), NULL);
	
	//End of melodic contour
	g_free(filepath1);
	#endif /*OFFICIAL_RELEASE */
	
	/* Cursor (modified from themeselection.c) */

	ClutterColor red = { 255, 0, 0, 128 };
	add_cursor(&red, sequencer, view);
}



void singinggame_cross_click(TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
/*
	const gchar* id_of_actor;
	id_of_actor = clutter_get_script_id(source);

	printf("cross_clicked. get_id: '%s' and parent '%s'\n",id_of_actor,clutter_get_script_id(G_OBJECT(clutter_actor_get_parent(CLUTTER_ACTOR(source)))));
*/

	//This is essential. If user press cross when mentor is talking, this sequencer_stop is handled _before_ sequencer_play
	jammo_mentor_shut_up(jammo_mentor_get_default());
	
	jammo_sequencer_stop(singinggame_sequencer);
}


void singinggame_start_songselection(){
	//Add transperency to songs which do not have current language vocal-track.
	//Hide advanced songs, when game level is easy.
	///////////////////////////////////////////////////////////////////////////////
	ClutterActor* container = jammo_get_actor_by_id("songselection-all-songs");
	const char* selected_language = chum_get_selected_language();
	GList* songlist;
	GList *l;


	songlist = clutter_container_get_children(CLUTTER_CONTAINER(container));
	for (l = songlist; l; l = l->next){
		const gchar* id_of_actor;
		id_of_actor = clutter_get_script_id(G_OBJECT(l->data));
		if (id_of_actor==NULL) continue; //In child-list there are many id=NULL actors.
			
		if (strncmp(id_of_actor,"songselection-song-",19)==0) {
			gchar* name;
			g_object_get(l->data, "name", &name, NULL);
			//printf("name of song: '%s'\n",name);
			
			ClutterActor* button;
			ClutterActor* texture;
			button = l->data;
			texture = tangle_button_get_normal_background_actor( TANGLE_BUTTON(button));


			gchar* filename;
			filename = g_strdup_printf("%s/songs/%s/vocal%s.ogg",DATA_DIR,name,selected_language);
			//printf("filename: '%s'\n",filename);
			int fd = open(filename,O_RDONLY);
			if (fd == -1  && chum_get_duetto() ) //File not found => no vocal_track for this language
				{
				//printf(" dim this\n");
				clutter_actor_set_opacity(texture,64); //0=transparency, 255=opaque
				}
			else
				{
				clutter_actor_set_opacity(texture,255); //0=transparency, 255=opaque
				}
				
			//Stupid way to mark these songs to advanced game, FIXME
			if (chum_is_easy_game() &&
					(strncmp(id_of_actor,"songselection-song-miska",24)==0 ||
					strncmp(id_of_actor,"songselection-song-pig",22)==0 ||
					strncmp(id_of_actor,"songselection-song-avignon",26)==0)  )
				{
				//printf("  hide this\n");
				clutter_actor_hide(button);
				}
			else
				{
				//printf("  show this\n");
				clutter_actor_show(button);
				}

			if (fd != -1) {
				close(fd);
			}
			g_free(filename);
			g_free(name);
		}
	}
	
	//Tune size of container
	if (chum_is_easy_game())
		clutter_actor_set_size(container, 670.0, 480); //Two rows
	else
		clutter_actor_set_size(container, 670.0, 600);  //Three rows

	g_list_free(songlist);
	
	if (!chum_is_easy_game()) //If game level is advanced, tell different instructions ( about scrolling )
		{ 
			jammo_mentor_speak_once(jammo_mentor_get_default(),"/opt/jammo/mentor_speech/mentor_songselection_welcome_advanced_fi.ogg");
		}
}



static gulong handler_for_mentor_say_selected;

static void mentor_say_what_is_selected(TangleActor *actor, ClutterActorBox *arg1, gpointer user_data){
	//printf("got '%s'\n",(gchar*)user_data);
	gchar *filepath1;
	filepath1 =  g_strdup_printf("/opt/jammo/mentor_speech/mentor%s_selected_fi.ogg", (gchar*)user_data);

	jammo_mentor_speak(jammo_mentor_get_default(), filepath1);
	g_free(filepath1);
	g_signal_handler_disconnect (actor,handler_for_mentor_say_selected);
}

void singinggame_duetto_clicked(TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
	ClutterActor* ct =CLUTTER_ACTOR(source);
	const gchar* name = clutter_actor_get_name(ct);
	printf("duetto clicked. get_name: '%s'\n",name);
	
	ClutterActor* view = jammo_get_actor_by_id("songselection-view");
	if (strcmp(name,"duetto")==0)
		{
		chum_set_duetto(TRUE);
		}
	else
		{
		chum_set_duetto(FALSE);
		}
	gchar *name_for_mentor;
	name_for_mentor =  g_strdup_printf("_%s", name);
	handler_for_mentor_say_selected=g_signal_connect_data(view, "show-completed", G_CALLBACK(mentor_say_what_is_selected),
                                                                  (gpointer)name_for_mentor, (GClosureNotify) g_free,0);
	
	ClutterActor* button;
	ClutterActor* texture;
	gchar *new_name;
	new_name = g_strdup_printf("songselection-%s-icon-texture",name);

	button = jammo_get_actor_by_id("songselection-duetto");
	texture = jammo_get_actor_by_id(new_name);
	tangle_button_set_normal_background_actor(TANGLE_BUTTON(button), texture);
	g_free(new_name);
}



void singinggame_language_clicked(TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
	ClutterActor* ct =CLUTTER_ACTOR(source);
	const gchar* name = clutter_actor_get_name(ct);
	printf("Language clicked. get_name: '%s'\n",name);

	chum_set_selected_language(name);

	ClutterActor* view = jammo_get_actor_by_id("songselection-view");
	handler_for_mentor_say_selected = g_signal_connect(view, "show-completed", G_CALLBACK(mentor_say_what_is_selected), (gpointer)name);
	
	ClutterActor* button;
	ClutterActor* texture;
	gchar *new_name = g_strdup_printf("songselection-language-%s-texture", name);
 
	button = jammo_get_actor_by_id("songselection-language");
	texture = jammo_get_actor_by_id(new_name);
	tangle_button_set_normal_background_actor(TANGLE_BUTTON(button), texture);
	g_free(new_name);
}



